// Common JS.

var _copyright_ = "Written by chown33.  Released into Public domain.";


// A global do-nothing function that can be assigned to any other method,
// or used any time a do-nothing function is desired.
function NOP() { };



// Global APP object holds all app-related funcs.
var APP = { };



// # # # #   U T I L I T I E S   # # # # 

// String Utilities.

// Trim leading and trailing whitespace from a string.
// For max portability using older browsers.
// In case JS doesn't have String.trim(), always use this func.
// Wikipedia says String.trim() is a built-in to JS 1.8.1, ECMAScript 5.
//
// Algorithm used here is trim12 implementation in this article:
//  http://blog.stevenlevithan.com/archives/faster-trim-javascript
//
APP.trim = function trim( str )  // trim12()
{
	var str = str.replace(/^\s\s*/, ''),
		ws = /\s/,
		i = str.length;
	while ( ws.test( str.charAt( --i ) ) );  // loop with no body
	return str.slice( 0, i + 1 );
}

// Split into lines, trimming whitespace from each item.
// Empty lines are stripped from output.
APP.toLines = function toLines( text )
{
	var lout = [ ];  // literal empty array
	var lin = text.split( "\n" );

	while ( lin.length > 0 )
	{
		var str = lin.shift();  // get from first
		if ( ! str )
			continue;

		str = APP.trim( str );

		// Comment out this line to keep empty lines in output.
		if ( str.length === 0 )  { continue; }

//		if ( str.startsWith( "#" ) )  // comment character
//			continue;

		lout.push( str );  // append to end
	}

	return lout;
}



// =====================================
//   "Console output" prints lines to an 'stdout' element.

// Default .show() and .clear() methods do nothing.
// These are given different functions during setup.
APP.show = function show()  { }
APP.clear = function clear()  { }  // clear previous stdout content

// Default .tip() method does nothing, which is APPEND mode.
// At any point, it can be assigned a different function that
// clears the stdout area, thus providing TRUNCATE mode.
// Since .clear() clears the stdout area, tip() either calls that or does nothing.
APP.tip = function tip()  { }

// Calls tip() then show().
APP.top = function top( str )  {  this.tip();  this.show( str ); }


// Setup the APP.show() method to output to given textArea, or to quietly do nothing.
// The textArea can be anything with a mutable 'value' property
// that can hold a multi-line string.
//   textArea - target object to serve as stdout text-container
APP.initShow = function initShow( textArea )  
{
	// Force both to do-nothing function
	APP.clear = APP.show = NOP;

	if ( textArea !== undefined  &&  textArea !== null )
	{
		APP.show = function _show( tx )  { textArea.value += "" + tx + "\n"; };  
		APP.clear = function _clear()  { textArea.value = ""; };  

		APP.tip = APP.clear;  // optional
	}
}


// an onload event handler for the body DOM-object.
APP.didLoad = function didLoad()  
{
	// We expect a textarea for the 'stdout' element, only by id.
	this.initShow( document.getElementById( 'stdout' ) );  // value may be undefined

	this.show( "Ready for input" );
}




// =====================================
//
// These are the parts of the app that change.


// Assume 'abc' is a series of values in "A,B,C" form,
// also allowing dots or colons instead of commas.
// Assume partD is a separate string.
// Assume all parts are URL-safe, and don't need percent-escaping.
APP.processOne = function processOne( abc, partD )  
{
	// Replace any dot or colon with comma, then split at commas.
	// Resulting parts is an array, possibly empty.
	var parts = abc.replace( /[:.]/g, ',' ).split( ',' );

	if ( parts.length < 3 )
	{
		this.show( "Too small: " + parts );
		return;
	}

	var partA = parts[ 0 ];
	var partB = parts[ 1 ];
	var partC = parts[ 2 ];

	this.show( 
		"www.example.com/someplace/inside"
		+ "/place" + partA + "/index.php?"
		+ "option1=" + partB
		+ "&option2=" + partC
		+ "&option3=" + partD );

}


// The 'lines' array is modified as it's processed.
// Each item processed is removed from front of array,
// so only unprocessed lines remain in array on return.
// Output is produced using APP.show().
APP.processLines = function processLines( lines )  
{
	// May hold text value of "param_D" element.
	var value_D = null;

	// Get the D parameter.  If none or empty, do nothing else.
	var param_D = document.getElementById( 'param_D' );  // value may be undefined
	if ( param_D )
		value_D = param_D.value;

	if ( ! value_D )
	{
		this.top( "Missing param D" );
		return;
	}

//	this.show( "Value of D: " + value_D );

	// Modifies 'lines' array as it's processed.
	while ( lines.length > 0 )
	{
		var str = lines.shift();  // get from front
		this.processOne( str, value_D );
	}
}


// Get lines from 'stdin' element (textarea),
// then call processLines() function to produce output in 'stdout' element.
APP.process = function process()  
{
	this.top( "Processing..." );

	// Start with empty array.
	var in_lines = [ ];

	// Check 'stdin' for lines of input.
	var input = document.getElementById( 'stdin' );  // value may be undefined
	if ( input )
	{  in_lines = this.toLines( input.value );  }

	if ( in_lines.length == 0 )
		this.top( "No input" );
	else
		this.processLines( in_lines );
}

